@@ -5,10 +5,11 @@ from visidata import *
 # see www/design/graphics.md
 
 option('show_graph_labels', True, 'show axes and legend on graph')
-option('plot_colors', 'green red yellow cyan magenta white 38 136 168', 'list of distinct colors to use for plotting distinct objects')
-option('disp_pixel_random', False, 'randomly choose attr from set of pixels instead of most common')
+theme('plot_colors', 'green red yellow cyan magenta white 38 136 168', 'list of distinct colors to use for plotting distinct objects')
+theme('disp_pixel_random', False, 'randomly choose attr from set of pixels instead of most common')
 option('zoom_incr', 2.0, 'amount to multiply current zoomlevel when zooming')
-option('color_graph_hidden', '238 blue', 'color of legend for hidden attribute')
+theme('color_graph_hidden', '238 blue', 'color of legend for hidden attribute')
+theme('color_graph_selected', 'bold', 'color of selected graph points')
 
 
 class Point:
@@ -16,7 +17,7 @@ class Point:
         self.x = x
         self.y = y
 
-    def __str__(self):
+    def __repr__(self):
         if isinstance(self.x, int):
             return '(%d,%d)' % (self.x, self.y)
         else:
@@ -33,6 +34,9 @@ class Box:
         self.w = w
         self.h = h
 
+    def __repr__(self):
+        return '[%s+%s,%s+%s]' % (self.xmin, self.w, self.ymin, self.h)
+
     @property
     def xymin(self):
         return Point(self.xmin, self.ymin)
@@ -131,40 +135,35 @@ class Plotter(BaseSheet):
     'pixel-addressable display of entire terminal with (x,y) integer pixel coordinates'
     columns=[Column('')]  # to eliminate errors outside of draw()
     rowtype='pixels'
-    commands=[
-        Command('^L', 'refresh()', 'redraw all pixels on canvas'),
-        Command('v', 'options.show_graph_labels = not options.show_graph_labels', 'toggle show_graph_labels'),
-        Command('KEY_RESIZE', 'refresh()', ''),
-    ]
-    def __init__(self, name, **kwargs):
+    def __init__(self, name='plotter', **kwargs):
         super().__init__(name, **kwargs)
         self.labels = []  # (x, y, text, attr, row)
         self.hiddenAttrs = set()
         self.needsRefresh = False
-        self.resetCanvasDimensions()
+        self.resetCanvasDimensions(vd.windowHeight, vd.windowWidth)
 
     def __len__(self):
         return (self.plotwidth* self.plotheight)
 
-    def resetCanvasDimensions(self):
-        'sets total available canvas dimensions'
-        self.plotwidth = vd().windowWidth*2
-        self.plotheight = (vd().windowHeight-1)*4  # exclude status line
+    def resetCanvasDimensions(self, windowHeight, windowWidth):
+        'sets total available canvas dimensions to (windowHeight, windowWidth) (in char cells)'
+        self.plotwidth = windowWidth*2
+        self.plotheight = (windowHeight-1)*4  # exclude status line
 
         # pixels[y][x] = { attr: list(rows), ... }
         self.pixels = [[defaultdict(list) for x in range(self.plotwidth)] for y in range(self.plotheight)]
 
-    def plotpixel(self, x, y, attr, row=None):
+    def plotpixel(self, x, y, attr=0, row=None):
         self.pixels[y][x][attr].append(row)
 
-    def plotline(self, x1, y1, x2, y2, attr, row=None):
+    def plotline(self, x1, y1, x2, y2, attr=0, row=None):
         for x, y in iterline(x1, y1, x2, y2):
             self.plotpixel(math.ceil(x), math.ceil(y), attr, row)
 
-    def plotlabel(self, x, y, text, attr, row=None):
+    def plotlabel(self, x, y, text, attr=0, row=None):
         self.labels.append((x, y, text, attr, row))
 
-    def plotlegend(self, i, txt, attr):
+    def plotlegend(self, i, txt, attr=0):
         self.plotlabel(self.plotwidth-30, i*4, txt, attr)
 
     @property
@@ -174,7 +173,10 @@ class Plotter(BaseSheet):
 
     @property
     def plotterMouse(self):
-        return Point(self.mouseX*2, self.mouseY*4)
+        return Point(*self.plotterFromTerminalCoord(self.mouseX, self.mouseY))
+
+    def plotterFromTerminalCoord(self, x, y):
+        return x*2, y*4
 
     def getPixelAttrRandom(self, x, y):
         'weighted-random choice of attr at this pixel.'
@@ -185,12 +187,12 @@ class Plotter(BaseSheet):
     def getPixelAttrMost(self, x, y):
         'most common attr at this pixel.'
         r = self.pixels[y][x]
-        c = sorted((len(rows), attr, rows) for attr, rows in r.items() if attr and attr not in self.hiddenAttrs)
+        c = sorted((len(rows), attr, rows) for attr, rows in list(r.items()) if attr and attr not in self.hiddenAttrs)
         if not c:
             return 0
         _, attr, rows = c[-1]
         if isinstance(self.source, BaseSheet) and anySelected(self.source, rows):
-            attr, _ = colors.update(attr, 8, 'bold', 10)
+            attr = CursesAttr(attr, 8).update_attr(colors.color_graph_selected, 10).attr
         return attr
 
     def hideAttr(self, attr, hide=True):
@@ -212,16 +214,18 @@ class Plotter(BaseSheet):
         return list(ret.values())
 
     def draw(self, scr):
+        windowHeight, windowWidth = scr.getmaxyx()
+
         if self.needsRefresh:
-            self.render()
+            self.render(windowHeight, windowWidth)
 
         scr.erase()
 
         if self.pixels:
             cursorBBox = self.plotterCursorBox
             getPixelAttr = self.getPixelAttrRandom if options.disp_pixel_random else self.getPixelAttrMost
-            for char_y in range(0, vd().windowHeight-1):  # save one line for status
-                for char_x in range(0, vd().windowWidth):
+            for char_y in range(0, self.plotheight//4):
+                for char_x in range(0, self.plotwidth//2):
                     block_attrs = [
                         getPixelAttr(char_x*2  , char_y*4  ),
                         getPixelAttr(char_x*2  , char_y*4+1),
@@ -247,7 +251,7 @@ class Plotter(BaseSheet):
 
                     if cursorBBox.contains(char_x*2, char_y*4) or \
                        cursorBBox.contains(char_x*2+1, char_y*4+3):
-                        attr, _ = colors.update(attr, 0, options.color_current_row, 10)
+                        attr = CursesAttr(attr).update_attr(colors.color_current_row).attr
 
                     if attr:
                         scr.addstr(char_y, char_x, chr(0x2800+braille_num), attr)
@@ -290,6 +294,8 @@ class Plotter(BaseSheet):
                         char_x, char_y, txt, attr, row = o
                         clipdraw(scr, char_y, char_x, txt, attr, len(txt))
 
+Plotter.addCommand('^L', 'redraw', 'refresh()')
+Plotter.addCommand('v', 'visibility', 'options.show_graph_labels = not options.show_graph_labels')
 
 # - has a cursor, of arbitrary position and width/height (not restricted to current zoom)
 class Canvas(Plotter):
@@ -297,67 +303,11 @@ class Canvas(Plotter):
     rowtype = 'plots'
     aspectRatio = 0.0
     leftMarginPixels = 10*2
-    rightMarginPixels = 6*2
+    rightMarginPixels = 4*2
     topMarginPixels = 0
     bottomMarginPixels = 1*4  # reserve bottom line for x axis
 
-    commands = Plotter.commands + [
-        Command('view-go-left', 'sheet.cursorBox.xmin -= cursorBox.w', ''),
-        Command('view-go-right', 'sheet.cursorBox.xmin += cursorBox.w', ''),
-        Command('view-go-up', 'sheet.cursorBox.ymin += cursorBox.h', ''),
-        Command('view-go-down', 'sheet.cursorBox.ymin -= cursorBox.h', ''),
-        Command('view-go-far-left', 'sheet.cursorBox.xmin = visibleBox.xmin', ''),
-        Command('view-go-far-right', 'sheet.cursorBox.xmin = visibleBox.xmax-cursorBox.w', ''),
-        Command('view-go-far-up', 'sheet.cursorBox.ymin = cursorBox.ymin', ''),
-        Command('view-go-far-down', 'sheet.cursorBox.ymin = cursorBox.ymax-cursorBox.h', ''),
-
-        Command('zh', 'sheet.cursorBox.xmin -= canvasCharWidth', ''),
-        Command('zl', 'sheet.cursorBox.xmin += canvasCharWidth', ''),
-        Command('zj', 'sheet.cursorBox.ymin += canvasCharHeight', ''),
-        Command('zk', 'sheet.cursorBox.ymin -= canvasCharHeight', ''),
-
-        Command('gH', 'sheet.cursorBox.w /= 2', ''),
-        Command('gL', 'sheet.cursorBox.w *= 2', ''),
-        Command('gJ', 'sheet.cursorBox.h /= 2', ''),
-        Command('gK', 'sheet.cursorBox.h *= 2', ''),
-
-        Command('H', 'sheet.cursorBox.w -= canvasCharWidth', ''),
-        Command('L', 'sheet.cursorBox.w += canvasCharWidth', ''),
-        Command('J', 'sheet.cursorBox.h += canvasCharHeight', ''),
-        Command('K', 'sheet.cursorBox.h -= canvasCharHeight', ''),
-
-        Command('zz', 'zoomTo(cursorBox)', 'set visible bounds to cursor'),
-
-        Command('-', 'tmp=cursorBox.center; setZoom(zoomlevel*options.zoom_incr); fixPoint(plotviewBox.center, tmp)', 'zoom out from cursor center'),
-        Command('+', 'tmp=cursorBox.center; setZoom(zoomlevel/options.zoom_incr); fixPoint(plotviewBox.center, tmp)', 'zoom into cursor center'),
-        Command('_', 'sheet.canvasBox = None; sheet.visibleBox = None; setZoom(1.0); refresh()', 'zoom to fit full extent'),
-        Command('z_', 'sheet.aspectRatio = float(input("aspect ratio=", value=aspectRatio)); refresh()', 'set aspect ratio'),
-
-        # set cursor box with left click
-        Command('BUTTON1_PRESSED', 'sheet.cursorBox = Box(*canvasMouse.xy)', 'start cursor box with left mouse button press'),
-        Command('BUTTON1_RELEASED', 'setCursorSize(canvasMouse)', 'end cursor box with left mouse button release'),
-
-        Command('BUTTON3_PRESSED', 'sheet.anchorPoint = canvasMouse', 'mark grid point to move'),
-        Command('BUTTON3_RELEASED', 'fixPoint(plotterMouse, anchorPoint)', 'mark canvas anchor point'),
-
-        Command('BUTTON4_PRESSED', 'tmp=canvasMouse; setZoom(zoomlevel/options.zoom_incr); fixPoint(plotterMouse, tmp)', 'zoom in with scroll wheel'),
-        Command('REPORT_MOUSE_POSITION', 'tmp=canvasMouse; setZoom(zoomlevel*options.zoom_incr); fixPoint(plotterMouse, tmp)', 'zoom out with scroll wheel'),
-
-        Command('s', 'source.select(list(rowsWithin(plotterCursorBox)))', 'select rows on source sheet contained within canvas cursor'),
-        Command('t', 'source.toggle(list(rowsWithin(plotterCursorBox)))', 'toggle selection of rows on source sheet contained within canvas cursor'),
-        Command('u', 'source.unselect(list(rowsWithin(plotterCursorBox)))', 'unselect rows on source sheet contained within canvas cursor'),
-        Command(ENTER, 'vs=copy(source); vs.rows=list(rowsWithin(plotterCursorBox)); vd.push(vs)', 'open sheet of source rows contained within canvas cursor'),
-        Command('d', 'source.delete(list(rowsWithin(plotterCursorBox))); reload()', 'delete rows on source sheet contained within canvas cursor'),
-
-
-        Command('gs', 'source.select(list(rowsWithin(plotterVisibleBox)))', 'select rows on source sheet visible on screen'),
-        Command('gt', 'source.toggle(list(rowsWithin(plotterVisibleBox)))', 'toggle selection of rows on source sheet visible on screen'),
-        Command('gu', 'source.unselect(list(rowsWithin(plotterVisibleBox)))', 'unselect rows on source sheet visible on screen'),
-        Command('g'+ENTER, 'vs=copy(source); vs.rows=list(rowsWithin(plotterVisibleBox)); vd.push(vs)', 'open sheet of source rows visible on screen'),
-        Command('gd', 'source.delete(list(rowsWithin(plotterVisibleBox))); reload()', 'delete rows on source sheet visible on screen'),
-    ]
-
-    def __init__(self, name, source=None, **kwargs):
+    def __init__(self, name='canvas', source=None, **kwargs):
         super().__init__(name, source=source, **kwargs)
 
         self.canvasBox = None   # bounding box of entire canvas, in canvas units
@@ -382,7 +332,7 @@ class Canvas(Plotter):
         self.polylines.clear()
         self.legends.clear()
         self.plotAttrs.clear()
-        self.unusedAttrs = list(colors[colorname.translate(str.maketrans('_', ' '))] for colorname in options.plot_colors.split())
+        self.unusedAttrs = list(colors[colorname.translate(str.maketrans('_', ' '))].attr for colorname in options.plot_colors.split())
 
     def plotColor(self, k):
         attr = self.plotAttrs.get(k, None)
@@ -399,8 +349,8 @@ class Canvas(Plotter):
             self.plotlegends()
         return attr
 
-    def resetCanvasDimensions(self):
-        super().resetCanvasDimensions()
+    def resetCanvasDimensions(self, windowHeight, windowWidth):
+        super().resetCanvasDimensions(windowHeight, windowWidth)
         self.plotviewBox = BoundingBox(self.leftMarginPixels, self.topMarginPixels,
                                        self.plotwidth-self.rightMarginPixels, self.plotheight-self.bottomMarginPixels-1)
 
@@ -410,8 +360,13 @@ class Canvas(Plotter):
 
     @property
     def canvasMouse(self):
-        return Point(self.visibleBox.xmin + (self.plotterMouse.x-self.plotviewBox.xmin)/self.xScaler,
-                     self.visibleBox.ymin + (self.plotterMouse.y-self.plotviewBox.ymin)/self.yScaler)
+        return self.canvasFromPlotterCoord(self.plotterMouse.x, self.plotterMouse.y)
+
+    def canvasFromPlotterCoord(self, plotter_x, plotter_y):
+        return Point(self.visibleBox.xmin + (plotter_x-self.plotviewBox.xmin)/self.xScaler, self.visibleBox.ymin + (plotter_y-self.plotviewBox.ymin)/self.yScaler)
+
+    def canvasFromTerminalCoord(self, x, y):
+        return self.canvasFromPlotterCoord(*self.plotterFromTerminalCoord(x, y))
 
     def setCursorSize(self, p):
         'sets width based on diagonal corner p'
@@ -445,21 +400,21 @@ class Canvas(Plotter):
                            self.scaleX(self.cursorBox.xmax),
                            self.scaleY(self.cursorBox.ymax))
 
-    def point(self, x, y, attr, row=None):
+    def point(self, x, y, attr=0, row=None):
         self.polylines.append(([(x, y)], attr, row))
 
-    def line(self, x1, y1, x2, y2, attr, row=None):
+    def line(self, x1, y1, x2, y2, attr=0, row=None):
         self.polylines.append(([(x1, y1), (x2, y2)], attr, row))
 
-    def polyline(self, vertexes, attr, row=None):
+    def polyline(self, vertexes, attr=0, row=None):
         'adds lines for (x,y) vertexes of a polygon'
         self.polylines.append((vertexes, attr, row))
 
-    def polygon(self, vertexes, attr, row=None):
+    def polygon(self, vertexes, attr=0, row=None):
         'adds lines for (x,y) vertexes of a polygon'
         self.polylines.append((vertexes + [vertexes[0]], attr, row))
 
-    def qcurve(self, vertexes, attr, row=None):
+    def qcurve(self, vertexes, attr=0, row=None):
         'quadratic curve from vertexes[0] to vertexes[2] with control point at vertexes[1]'
         assert len(vertexes) == 3, len(vertexes)
         x1, y1 = vertexes[0]
@@ -524,7 +479,7 @@ class Canvas(Plotter):
         self._recursive_bezier(x1, y1, x12, y12, x123, y123, attr, row, level + 1)
         self._recursive_bezier(x123, y123, x23, y23, x3, y3, attr, row, level + 1)
 
-    def label(self, x, y, text, attr, row=None):
+    def label(self, x, y, text, attr=0, row=None):
         self.gridlabels.append((x, y, text, attr, row))
 
     def fixPoint(self, plotterPoint, canvasPoint):
@@ -554,7 +509,7 @@ class Canvas(Plotter):
                     if ymin is None or y < ymin: ymin = y
                     if xmax is None or x > xmax: xmax = x
                     if ymax is None or y > ymax: ymax = y
-            self.canvasBox = BoundingBox(xmin or 0, ymin or 0, xmax or 1, ymax or 1)
+            self.canvasBox = BoundingBox(float(xmin or 0), float(ymin or 0), float(xmax or 1), float(ymax or 1))
 
         if not self.visibleBox:
             # initialize minx/miny, but w/h must be set first to center properly
@@ -571,13 +526,19 @@ class Canvas(Plotter):
     def plotlegends(self):
         # display labels
         for i, (legend, attr) in enumerate(self.legends.items()):
-            self._commands[str(i+1)] = Command(str(i+1), 'hideAttr(%s, %s not in hiddenAttrs)' % (attr, attr), 'toggle display of "%s"' % legend)
+            self.addCommand(str(i+1), 'toggle-%s'%(i+1), 'hideAttr(%s, %s not in hiddenAttrs)' % (attr, attr)) #, 'toggle display of "%s"' % legend)
             if attr in self.hiddenAttrs:
-                attr = colors[options.color_graph_hidden]
+                attr = colors.color_graph_hidden
             self.plotlegend(i, '%s:%s'%(i+1,legend), attr)
 
     def checkCursor(self):
         'override Sheet.checkCursor'
+        if self.cursorBox:
+            if self.cursorBox.h < self.canvasCharHeight:
+                self.cursorBox.h = self.canvasCharHeight*3/4
+            if self.cursorBox.w < self.canvasCharWidth:
+                self.cursorBox.w = self.canvasCharWidth*3/4
+
         return False
 
     @property
@@ -618,16 +579,19 @@ class Canvas(Plotter):
         'triggers render() on next draw()'
         self.needsRefresh = True
 
-    def render(self):
+    def render(self, h, w):
         'resets plotter, cancels previous render threads, spawns a new render'
         self.needsRefresh = False
         cancelThread(*(t for t in self.currentThreads if t.name == 'plotAll_async'))
         self.labels.clear()
-        self.resetCanvasDimensions()
+        self.resetCanvasDimensions(h, w)
         self.render_async()
 
     @asyncthread
     def render_async(self):
+        self.render_sync()
+
+    def render_sync(self):
         'plots points and lines and text onto the Plotter'
 
         self.setZoom()
@@ -636,9 +600,10 @@ class Canvas(Plotter):
         xfactor, yfactor = self.xScaler, self.yScaler
         plotxmin, plotymin = self.plotviewBox.xmin, self.plotviewBox.ymin
 
-        for vertexes, attr, row in Progress(self.polylines):
+        for vertexes, attr, row in Progress(self.polylines, 'rendering'):
             if len(vertexes) == 1:  # single point
                 x1, y1 = vertexes[0]
+                x1, y1 = float(x1), float(y1)
                 if xmin <= x1 <= xmax and ymin <= y1 <= ymax:
                     x = plotxmin+(x1-xmin)*xfactor
                     y = plotymin+(y1-ymin)*yfactor
@@ -650,13 +615,68 @@ class Canvas(Plotter):
                 r = clipline(prev_x, prev_y, x, y, xmin, ymin, xmax, ymax)
                 if r:
                     x1, y1, x2, y2 = r
-                    x1 = plotxmin+(x1-xmin)*xfactor
-                    y1 = plotymin+(y1-ymin)*yfactor
-                    x2 = plotxmin+(x2-xmin)*xfactor
-                    y2 = plotymin+(y2-ymin)*yfactor
+                    x1 = plotxmin+float(x1-xmin)*xfactor
+                    y1 = plotymin+float(y1-ymin)*yfactor
+                    x2 = plotxmin+float(x2-xmin)*xfactor
+                    y2 = plotymin+float(y2-ymin)*yfactor
                     self.plotline(x1, y1, x2, y2, attr, row)
                 prev_x, prev_y = x, y
 
-        for x, y, text, attr, row in Progress(self.gridlabels):
+        for x, y, text, attr, row in Progress(self.gridlabels, 'labeling'):
             self.plotlabel(self.scaleX(x), self.scaleY(y), text, attr, row)
 
+Canvas.addCommand(None, 'go-left', 'sheet.cursorBox.xmin -= cursorBox.w')
+Canvas.addCommand(None, 'go-right', 'sheet.cursorBox.xmin += cursorBox.w')
+Canvas.addCommand(None, 'go-up', 'sheet.cursorBox.ymin -= cursorBox.h')
+Canvas.addCommand(None, 'go-down', 'sheet.cursorBox.ymin += cursorBox.h')
+Canvas.addCommand(None, 'go-leftmost', 'sheet.cursorBox.xmin = visibleBox.xmin')
+Canvas.addCommand(None, 'go-rightmost', 'sheet.cursorBox.xmin = visibleBox.xmax-cursorBox.w')
+Canvas.addCommand(None, 'go-top', 'sheet.cursorBox.ymin = visibleBox.ymin')
+Canvas.addCommand(None, 'go-bottom', 'sheet.cursorBox.ymin = visibleBox.ymax')
+
+Canvas.addCommand(None, 'next-page', 't=(visibleBox.ymax-visibleBox.ymin); sheet.cursorBox.ymin += t; sheet.visibleBox.ymin += t; refresh()')
+Canvas.addCommand(None, 'prev-page', 't=(visibleBox.ymax-visibleBox.ymin); sheet.cursorBox.ymin -= t; sheet.visibleBox.ymin -= t; refresh()')
+
+Canvas.addCommand('zh', 'go-left-small', 'sheet.cursorBox.xmin -= canvasCharWidth')
+Canvas.addCommand('zl', 'go-right-small', 'sheet.cursorBox.xmin += canvasCharWidth')
+Canvas.addCommand('zj', 'go-down-small', 'sheet.cursorBox.ymin += canvasCharHeight')
+Canvas.addCommand('zk', 'go-up-small', 'sheet.cursorBox.ymin -= canvasCharHeight')
+
+Canvas.addCommand('gH', 'resize-cursor-halfwide', 'sheet.cursorBox.w /= 2')
+Canvas.addCommand('gL', 'resize-cursor-doublewide', 'sheet.cursorBox.w *= 2')
+Canvas.addCommand('gJ','resize-cursor-halfheight', 'sheet.cursorBox.h /= 2')
+Canvas.addCommand('gK', 'resize-cursor-doubleheight', 'sheet.cursorBox.h *= 2')
+
+Canvas.addCommand('H', 'resize-cursor-thinner', 'sheet.cursorBox.w -= canvasCharWidth')
+Canvas.addCommand('L', 'resize-cursor-wider', 'sheet.cursorBox.w += canvasCharWidth')
+Canvas.addCommand('J', 'resize-cursor-taller', 'sheet.cursorBox.h += canvasCharHeight')
+Canvas.addCommand('K', 'resize-cursor-shorter', 'sheet.cursorBox.h -= canvasCharHeight')
+Canvas.addCommand('zz', 'zoom-cursor', 'zoomTo(cursorBox)')
+
+Canvas.addCommand('-', 'zoomout-cursor', 'tmp=cursorBox.center; setZoom(zoomlevel*options.zoom_incr); fixPoint(plotviewBox.center, tmp)')
+Canvas.addCommand('+', 'zoomin-cursor', 'tmp=cursorBox.center; setZoom(zoomlevel/options.zoom_incr); fixPoint(plotviewBox.center, tmp)')
+Canvas.addCommand('_', 'zoom-all', 'sheet.canvasBox = None; sheet.visibleBox = None; setZoom(1.0); refresh()')
+Canvas.addCommand('z_', 'set-aspect', 'sheet.aspectRatio = float(input("aspect ratio=", value=aspectRatio)); refresh()')
+
+# set cursor box with left click
+Canvas.addCommand('BUTTON1_PRESSED', 'start-cursor', 'sheet.cursorBox = Box(*canvasMouse.xy)')
+Canvas.addCommand('BUTTON1_RELEASED', 'end-cursor', 'setCursorSize(canvasMouse)')
+
+Canvas.addCommand('BUTTON3_PRESSED', 'start-move', 'sheet.anchorPoint = canvasMouse')
+Canvas.addCommand('BUTTON3_RELEASED', 'end-move', 'fixPoint(plotterMouse, anchorPoint)')
+
+Canvas.addCommand('BUTTON4_PRESSED', 'zoomin-mouse', 'tmp=canvasMouse; setZoom(zoomlevel/options.zoom_incr); fixPoint(plotterMouse, tmp)')
+Canvas.addCommand('REPORT_MOUSE_POSITION', 'zoomout-mouse', 'tmp=canvasMouse; setZoom(zoomlevel*options.zoom_incr); fixPoint(plotterMouse, tmp)')
+
+Canvas.addCommand('s', 'select-cursor', 'source.select(list(rowsWithin(plotterCursorBox)))')
+Canvas.addCommand('t', 'stoggle-cursor', 'source.toggle(list(rowsWithin(plotterCursorBox)))')
+Canvas.addCommand('u', 'unselect-cursor', 'source.unselect(list(rowsWithin(plotterCursorBox)))')
+Canvas.addCommand(ENTER, 'dive-cursor', 'vs=copy(source); vs.rows=list(rowsWithin(plotterCursorBox)); vd.push(vs)')
+Canvas.addCommand('d', 'delete-cursor', 'source.delete(list(rowsWithin(plotterCursorBox))); reload()')
+
+
+Canvas.addCommand('gs', 'select-visible', 'source.select(list(rowsWithin(plotterVisibleBox)))')
+Canvas.addCommand('gt', 'stoggle-visible', 'source.toggle(list(rowsWithin(plotterVisibleBox)))')
+Canvas.addCommand('gu', 'unselect-visible', 'source.unselect(list(rowsWithin(plotterVisibleBox)))')
+Canvas.addCommand('g'+ENTER, 'dive-visible', 'vs=copy(source); vs.rows=list(rowsWithin(plotterVisibleBox)); vd.push(vs)')
+Canvas.addCommand('gd', 'delete-visible', 'source.delete(list(rowsWithin(plotterVisibleBox))); reload()')
