@@ -2,8 +2,8 @@ from visidata import *
 
 option('color_graph_axis', 'bold', 'color for graph axis labels')
 
-globalCommand('.', 'vd.push(GraphSheet(sheet.name+"_graph", sheet, selectedRows or rows, keyCols, [cursorCol]))', 'graph the current column vs key columns Numeric key column is on the x-axis, while categorical key columns determin color', 'data-plot-column')
-globalCommand('g.', 'vd.push(GraphSheet(sheet.name+"_graph", sheet, selectedRows or rows, keyCols, numericCols(nonKeyVisibleCols)))', 'open a graph of all visible numeric columns vs key column', 'data-plot-allnumeric')
+Sheet.addCommand('.', 'plot-column', 'vd.push(GraphSheet(sheet.name+"_graph", sheet, rows, keyCols, [cursorCol]))')
+Sheet.addCommand('g.', 'plot-numerics', 'vd.push(GraphSheet(sheet.name+"_graph", sheet, rows, keyCols, numericCols(nonKeyVisibleCols)))')
 
 
 def numericCols(cols):
@@ -12,18 +12,6 @@ def numericCols(cols):
 
 
 class InvertedCanvas(Canvas):
-    commands = Canvas.commands + [
-        # swap directions of up/down
-        Command('move-up', 'sheet.cursorBox.ymin += cursorBox.h', 'move cursor up'),
-        Command('move-down', 'sheet.cursorBox.ymin -= cursorBox.h', 'move cursor down'),
-
-        Command('zj', 'sheet.cursorBox.ymin -= canvasCharHeight', 'move cursor down one line'),
-        Command('zk', 'sheet.cursorBox.ymin += canvasCharHeight', 'move cursor up one line'),
-
-        Command('J', 'sheet.cursorBox.h -= canvasCharHeight', 'decrease cursor height'),
-        Command('K', 'sheet.cursorBox.h += canvasCharHeight', 'increase cursor height'),
-    ]
-
     def zoomTo(self, bbox):
         super().zoomTo(bbox)
         self.fixPoint(Point(self.plotviewBox.xmin, self.plotviewBox.ymax), bbox.xymin)
@@ -46,6 +34,20 @@ class InvertedCanvas(Canvas):
         p.y = self.visibleBox.ymin + (self.plotviewBox.ymax-self.plotterMouse.y)/self.yScaler
         return p
 
+# swap directions of up/down
+InvertedCanvas.addCommand(None, 'go-up', 'sheet.cursorBox.ymin += cursorBox.h')
+InvertedCanvas.addCommand(None, 'go-down', 'sheet.cursorBox.ymin -= cursorBox.h')
+InvertedCanvas.addCommand(None, 'go-top', 'sheet.cursorBox.ymin = visibleBox.ymax')
+InvertedCanvas.addCommand(None, 'go-bottom', 'sheet.cursorBox.ymin = visibleBox.ymin')
+InvertedCanvas.addCommand(None, 'next-page', 't=(visibleBox.ymax-visibleBox.ymin); sheet.cursorBox.ymin -= t; sheet.visibleBox.ymin -= t; refresh()')
+InvertedCanvas.addCommand(None, 'prev-page', 't=(visibleBox.ymax-visibleBox.ymin); sheet.cursorBox.ymin += t; sheet.visibleBox.ymin += t; refresh()')
+
+InvertedCanvas.addCommand(None, 'go-down-small', 'sheet.cursorBox.ymin -= canvasCharHeight')
+InvertedCanvas.addCommand(None, 'go-up-small', 'sheet.cursorBox.ymin += canvasCharHeight')
+
+InvertedCanvas.addCommand(None, 'resize-cursor-shorter', 'sheet.cursorBox.h -= canvasCharHeight')
+InvertedCanvas.addCommand(None, 'resize-cursor-taller', 'sheet.cursorBox.h += canvasCharHeight')
+
 
 # provides axis labels, legend
 class GraphSheet(InvertedCanvas):
@@ -53,7 +55,7 @@ class GraphSheet(InvertedCanvas):
         super().__init__(name, sheet, sourceRows=rows, **kwargs)
 
         self.xcols = xcols
-        self.ycols = [ycol for ycol in ycols if isNumeric(ycol)] or error('%s is non-numeric' % '/'.join(yc.name for yc in ycols))
+        self.ycols = [ycol for ycol in ycols if isNumeric(ycol)] or fail('%s is non-numeric' % '/'.join(yc.name for yc in ycols))
 
     @asyncthread
     def reload(self):
@@ -66,7 +68,7 @@ class GraphSheet(InvertedCanvas):
         catcols = [c for c in self.xcols if not isNumeric(c)]
         numcols = numericCols(self.xcols)
         for ycol in self.ycols:
-            for rownum, row in enumerate(Progress(self.sourceRows)):  # rows being plotted from source
+            for rownum, row in enumerate(Progress(self.sourceRows, 'plotting')):  # rows being plotted from source
                 try:
                     k = tuple(c.getValue(row) for c in catcols) if catcols else (ycol.name,)
 
@@ -94,15 +96,11 @@ class GraphSheet(InvertedCanvas):
 
     def add_y_axis_label(self, frac):
         amt = self.visibleBox.ymin + frac*self.visibleBox.h
-        if isinstance(self.canvasBox.ymin, int):
-            txt = '%d' % amt
-        elif isinstance(self.canvasBox.ymin, float):
-            txt = '%.02f' % amt
-        else:
-            txt = str(frac)
+        srccol = self.ycols[0]
+        txt = srccol.format(srccol.type(amt))
 
         # plot y-axis labels on the far left of the canvas, but within the plotview height-wise
-        attr = colors[options.color_graph_axis]
+        attr = colors.color_graph_axis
         self.plotlabel(0, self.plotviewBox.ymin + (1.0-frac)*self.plotviewBox.h, txt, attr)
 
     def add_x_axis_label(self, frac):
@@ -110,8 +108,13 @@ class GraphSheet(InvertedCanvas):
         txt = ','.join(xcol.format(xcol.type(amt)) for xcol in self.xcols if isNumeric(xcol))
 
         # plot x-axis labels below the plotviewBox.ymax, but within the plotview width-wise
-        attr = colors[options.color_graph_axis]
-        self.plotlabel(self.plotviewBox.xmin+frac*self.plotviewBox.w, self.plotviewBox.ymax+4, txt, attr)
+        attr = colors.color_graph_axis
+        xmin = self.plotviewBox.xmin + frac*self.plotviewBox.w
+        if frac == 1.0:
+            # shift rightmost label to be readable
+            xmin -= max(len(txt)*2 - self.rightMarginPixels+1, 0)
+
+        self.plotlabel(xmin, self.plotviewBox.ymax+4, txt, attr)
 
     def createLabels(self):
         self.gridlabels = []
@@ -134,4 +137,4 @@ class GraphSheet(InvertedCanvas):
         # TODO: grid lines corresponding to axis labels
 
         xname = ','.join(xcol.name for xcol in self.xcols if isNumeric(xcol)) or 'row#'
-        self.plotlabel(0, self.plotviewBox.ymax+4, '%*s»' % (int(self.leftMarginPixels/2-2), xname), colors[options.color_graph_axis])
+        self.plotlabel(0, self.plotviewBox.ymax+4, '%*s»' % (int(self.leftMarginPixels/2-2), xname), colors.color_graph_axis)
