@@ -15,7 +15,8 @@ class FixedWidthColumn(Column):
         return row[0][self.i:self.j]
 
     def setValue(self, row, value):
-        row[0] = row[:self.i] + '%*s' % (self.j-self.i, value) + row[self.j:]
+        value = str(value)[:self.j-self.i]
+        row[0] = row[0][:self.i] + '%-*s' % (self.j-self.i, value) + row[0][self.j:]
 
 def columnize(rows):
     'Generate (i,j) indexes for fixed-width columns found in rows'
