@@ -28,26 +28,19 @@ class ShapeSheet(Sheet):
     columns = [
         Column('shapeType', width=0, getter=lambda col,row: row.shape.shapeType)
     ]
-    commands = [
-        Command('.', 'vd.push(ShapeMap(name+"_map", sheet, sourceRows=[cursorRow], textCol=cursorCol))', ''),
-        Command('g.', 'vd.push(ShapeMap(name+"_map", sheet, sourceRows=selectedRows or rows, textCol=cursorCol))', ''),
-    ]
     @asyncthread
     def reload(self):
         import shapefile
         sf = shapefile.Reader(self.source.resolve())
+        self.columns = copy(ShapeSheet.columns)
         for i, (fname, ftype, fieldlen, declen) in enumerate(sf.fields[1:]):  # skip DeletionFlag
             self.addColumn(Column(fname, getter=lambda col,row,i=i: row.record[i], type=shptype(ftype, declen)))
         self.rows = []
         for shaperec in Progress(sf.iterShapeRecords(), total=sf.numRecords):
             self.addRow(shaperec)
 
-
 class ShapeMap(InvertedCanvas):
     aspectRatio = 1.0
-    commands = [
-        Command('^S', 'save_geojson(Path(input("json to save: ", value=name+".geojson")), sheet)', 'save in geojson format'),
-    ]
     @asyncthread
     def reload(self):
         self.reset()
@@ -73,12 +66,16 @@ class ShapeMap(InvertedCanvas):
 
         self.refresh()
 
+ShapeSheet.addCommand('.', 'plot-row', 'vd.push(ShapeMap(name+"_map", sheet, sourceRows=[cursorRow], textCol=cursorCol))')
+ShapeSheet.addCommand('g.', 'plot-selected', 'vd.push(ShapeMap(name+"_map", sheet, sourceRows=selectedRows or rows, textCol=cursorCol))')
+ShapeMap.addCommand('^S', 'save-geojson', 'save_geojson(Path(input("json to save: ", value=name+".geojson")), sheet)')
+
 
 def save_geojson(p, vs):
     assert isinstance(vs, Canvas), 'need Canvas to save geojson'
 
     features = []
-    for coords, attr, row in Progress(vs.polylines):
+    for coords, attr, row in Progress(vs.polylines, 'saving'):
         feat = {
             'type': 'Feature',
             'geometry': {
