@@ -16,9 +16,7 @@ class xlsxContents(Sheet):
         ColumnAttr('nCols', type=int),
     ]
     nKeys = 1
-    commands = [
-        Command(ENTER, 'vd.push(cursorRow)', 'load the entire table into memory')
-    ]
+
     def __init__(self, path):
         super().__init__(path.name, source=path)
         self.workbook = None
@@ -33,6 +31,7 @@ class xlsxContents(Sheet):
             vs.reload()
             self.rows.append(vs)
 
+xlsxContents.addCommand(ENTER, 'dive-row', 'vd.push(cursorRow)')
 
 class xlsxSheet(Sheet):
     @asyncthread
@@ -41,19 +40,26 @@ class xlsxSheet(Sheet):
 
         self.columns = []
         self.rows = []
-        for row in Progress(worksheet.iter_rows(), worksheet.max_row or 0):
-            L = list(cell.value for cell in row)
-            for i in range(len(self.columns), len(L)):  # no-op if already done
+
+        rows = worksheet.iter_rows()
+        hdrs = [list(wrapply(getattr, cell, 'value')
+                   for cell in next(rows))
+                       for i in range(options.header)
+               ]
+        colnames = ['\n'.join(str(hdr[i]) for i in range(len(hdr))) for hdr in zip(*hdrs)]
+        for i, colname in enumerate(colnames):
+            self.addColumn(ColumnItem(colname, i))
+
+        for r in Progress(rows, total=worksheet.max_row or 0):
+            row = list(wrapply(getattr, cell, 'value') for cell in r)
+            for i in range(len(self.columns), len(row)):  # no-op if already done
                 self.addColumn(ColumnItem(None, i, width=8))
-            self.addRow(L)
+            self.addRow(row)
 
 
 class open_xls(Sheet):
     'Load XLS file (in Excel format).'
     rowtype = 'sheets'  # rowdef: xlsSheet
-    commands = [
-        Command(ENTER, 'vd.push(cursorRow)', 'load the entire table into memory')
-    ]
     columns = [
         Column('sheet', getter=lambda col,row: row.source.name),  # xls sheet name
         ColumnAttr('name', width=0),  # visidata sheet name
@@ -75,15 +81,23 @@ class open_xls(Sheet):
             vs.reload()
             self.rows.append(vs)
 
+open_xls.addCommand(ENTER, 'dive-row', 'vd.push(cursorRow)')
 
 class xlsSheet(Sheet):
     @asyncthread
     def reload(self):
         worksheet = self.source
         self.columns = []
-        for i in range(worksheet.ncols):
-            self.addColumn(ColumnItem(None, i, width=8))
+        if options.header:
+            hdrs = [list(worksheet.cell(rownum, colnum).value for colnum in range(worksheet.ncols))
+                        for rownum in range(options.header)]
+            colnames = ['\\n'.join(str(hdr[i]) for i in range(len(hdr))) for hdr in zip(*hdrs)]
+        else:
+            colnames = ['']*worksheet.ncols
+
+        for i, colname in enumerate(colnames):
+            self.addColumn(ColumnItem(colname, i))
 
         self.rows = []
-        for rownum in Progress(range(worksheet.nrows)):
+        for rownum in Progress(range(options.header, worksheet.nrows)):
             self.addRow(list(worksheet.cell(rownum, colnum).value for colnum in range(worksheet.ncols)))
