@@ -1,25 +1,61 @@
 from visidata import *
 
-option('pyobj_show_hidden', False, 'show _private properties on pyobjs')
-option('pyobj_show_methods', False, 'show methods on pyobjs')
+option('visibility', 0, 'visibility level')
 
-globalCommand('^X', 'expr = input("eval: ", "expr", completer=CompleteExpr()); push_pyobj(expr, evalexpr(expr, cursorRow))', 'evaluate Python expression and open result as Python object', 'python-eval-push')
-globalCommand('g^X', 'expr = input("exec: ", "expr", completer=CompleteExpr()); exec(expr, getGlobals())', 'execute Python statement in the global scope', 'python-exec')
-globalCommand('z^X', 'status(evalexpr(inputExpr("status="), cursorRow))', 'evaluate Python expression on current row and show result on status line', 'python-eval-status')
+globalCommand('^X', 'pyobj-expr', 'expr = input("eval: ", "expr", completer=CompleteExpr()); push_pyobj(expr, evalexpr(expr, None))')
+globalCommand('g^X', 'exec-python', 'expr = input("exec: ", "expr", completer=CompleteExpr()); exec(expr, getGlobals())')
+globalCommand('z^X', 'pyobj-expr-row', 'expr = input("eval over current row: ", "expr", completer=CompleteExpr()); push_pyobj(expr, evalexpr(expr, cursorRow))')
 
-globalCommand('^Y', 'status(type(cursorRow)); push_pyobj("%s[%s]" % (sheet.name, cursorRowIndex), cursorRow)', 'open current row as Python object', 'python-push-row-object')
-globalCommand('z^Y', 'status(type(cursorValue)); push_pyobj("%s[%s].%s" % (sheet.name, cursorRowIndex, cursorCol.name), cursorValue)', 'open current cell as Python object', 'python-push-cell-object')
-globalCommand('g^Y', 'status(type(sheet)); push_pyobj(sheet.name+"_sheet", sheet)', 'open current sheet as Python object', 'python-push-sheet-object')
+Sheet.addCommand('^Y', 'pyobj-row', 'status(type(cursorRow)); push_pyobj("%s[%s]" % (sheet.name, cursorRowIndex), cursorRow)')
+Sheet.addCommand('z^Y', 'pyobj-cell', 'status(type(cursorValue)); push_pyobj("%s[%s].%s" % (sheet.name, cursorRowIndex, cursorCol.name), cursorValue)')
+globalCommand('g^Y', 'pyobj-sheet', 'status(type(sheet)); push_pyobj(sheet.name+"_sheet", sheet)')
 
-globalCommand('(', 'openColumn(cursorCol, cursorValue)', 'expand current column of lists or dict into multiple columns and hide the original column')
-globalCommand(')', 'closeColumn(sheet, cursorCol)', 'unexpand current column; restore original column and remove other columns at this level')
+Sheet.addCommand('(', 'expand-col', 'expand_cols_deep(sheet, [cursorCol], cursorRow, depth=0)')
+Sheet.addCommand('g(', 'expand-cols', 'expand_cols_deep(sheet, visibleCols, cursorRow, depth=0)')
+Sheet.addCommand('z(', 'expand-col-depth', 'expand_cols_deep(sheet, [cursorCol], cursorRow, depth=int(input("expand depth=", value=1)))')
+Sheet.addCommand('gz(', 'expand-cols-depth', 'expand_cols_deep(sheet, visibleCols, cursorRow, depth=int(input("expand depth=", value=1)))')
+
+Sheet.addCommand(')', 'contract-col', 'closeColumn(sheet, cursorCol)')
+
+class PythonSheet(Sheet):
+    pass
 
 # used as ENTER in several pyobj sheets
-globalCommand('python-dive-row', 'push_pyobj("%s[%s]" % (name, cursorRowIndex), cursorRow).cursorRowIndex = cursorColIndex', 'dive further into Python object')
+PythonSheet.addCommand(None, 'dive-row', 'push_pyobj("%s[%s]" % (name, cursorRowIndex), cursorRow)')
+bindkey(ENTER, 'dive-row')
+
+def expand_cols_deep(sheet, cols, row, depth=0):  # depth == 0 means drill all the way
+    'expand all visible columns of containers to the given depth (0=fully)'
+    ret = []
+    for col in cols:
+        newcols = _addExpandedColumns(col, row, sheet.columns.index(col))
+        if depth != 1:  # countdown not yet complete, or negative (indefinite)
+            ret.extend(expand_cols_deep(sheet, newcols, row, depth-1))
+    return ret
+
+def _addExpandedColumns(col, row, idx):
+    val = col.getTypedValueNoExceptions(row)
+    if isinstance(val, dict):
+        ret = [
+            ExpandedColumn('%s.%s' % (col.name, k), type=deduceType(val[k]), origCol=col, key=k)
+                for k in val
+        ]
+    elif isinstance(val, (list, tuple)):
+        ret = [
+            ExpandedColumn('%s[%s]' % (col.name, k), type=deduceType(val[k]), origCol=col, key=k)
+                for k in range(len(val))
+        ]
+    else:
+        return []
+
+    for i, c in enumerate(ret):
+        col.sheet.addColumn(c, idx+i+1)
+    col.hide()
+    return ret
 
 
 def deduceType(v):
-    if isinstance(v, (float,int,str)):
+    if isinstance(v, (float, int)):
         return type(v)
     else:
         return anytype
@@ -27,22 +63,11 @@ def deduceType(v):
 
 class ExpandedColumn(Column):
     def calcValue(self, row):
-        return self.origCol.getValue(row)[self.key]
+        return getitemdef(self.origCol.getValue(row), self.key)
 
     def setValue(self, row, value):
         self.origCol.getValue(row)[self.key] = value
 
-def openColumn(col, val):
-    if isinstance(val, dict):
-        for k in val:
-            c = ExpandedColumn(col.name+'.'+k, type=deduceType(val[k]), origCol=col, key=k)
-            col.sheet.addColumn(c, col.sheet.cursorColIndex+1)
-            col.width = 0
-    elif isinstance(val, (list, tuple)):
-        for k in range(len(val)):
-            c = ExpandedColumn('%s[%s]' % (col.name, k), type=deduceType(val[k]), origCol=col, key=k)
-            col.sheet.addColumn(c, col.sheet.cursorColIndex+1)
-            col.width = 0
 
 def closeColumn(sheet, col):
     col.origCol.width = options.default_width
@@ -50,13 +75,17 @@ def closeColumn(sheet, col):
     sheet.columns = cols
 
 
+
 #### generic list/dict/object browsing
 def push_pyobj(name, pyobj):
     vs = load_pyobj(name, pyobj)
     if vs:
         return vd().push(vs)
     else:
-        status('unknown type ' + type(pyobj))
+        error("cannot push '%s' as pyobj" % type(pyobj).__name__)
+
+def view(obj):
+    run(load_pyobj(getattr(obj, '__name__', ''), obj))
 
 def load_pyobj(name, pyobj):
     'Return Sheet object of appropriate type for given sources in `args`.'
@@ -70,7 +99,7 @@ def load_pyobj(name, pyobj):
     elif isinstance(pyobj, object):
         return SheetObject(name, pyobj)
     else:
-        status('unknown type ' + type(pyobj))
+        error("cannot load '%s' as pyobj" % type(pyobj).__name__)
 
 def open_pyobj(path):
     'Provide wrapper for `load_pyobj`.'
@@ -108,9 +137,8 @@ def SheetList(name, src, **kwargs):
     # simple list
     return ListOfPyobjSheet(name, source=src, **kwargs)
 
-class ListOfPyobjSheet(Sheet):
+class ListOfPyobjSheet(PythonSheet):
     rowtype = 'python objects'
-    commands = [Command(ENTER, 'python-dive-row')]
     def reload(self):
         self.rows = self.source
         self.columns = [Column(self.name,
@@ -118,27 +146,24 @@ class ListOfPyobjSheet(Sheet):
                                setter=lambda col,row,val: setitem(col.sheet.source, col.sheet.source.index(row), val))]
 
 # rowdef: dict
-class ListOfDictSheet(Sheet):
+class ListOfDictSheet(PythonSheet):
     rowtype = 'dicts'
-    commands = [Command(ENTER, 'python-dive-row')]
     def reload(self):
         self.columns = DictKeyColumns(self.source[0])
         self.rows = self.source
 
 # rowdef: namedtuple
-class ListOfNamedTupleSheet(Sheet):
+class ListOfNamedTupleSheet(PythonSheet):
     rowtype = 'namedtuples'
-    commands = [Command(ENTER, 'python-dive-row')]
     def reload(self):
         self.columns = [ColumnItem(k, i) for i, k in enumerate(self.source[0]._fields)]
         self.rows = self.source
 
 
 # rowdef: PyObj
-class SheetNamedTuple(Sheet):
-    rowtype = 'values'
+class SheetNamedTuple(PythonSheet):
     'a single namedtuple, with key and value columns'
-    commands = [Command(ENTER, 'dive()', 'dive further into Python object', 'python-dive-row')]
+    rowtype = 'values'
     columns = [ColumnItem('name', 0), ColumnItem('value', 1)]
 
     def __init__(self, name, src, **kwargs):
@@ -150,15 +175,12 @@ class SheetNamedTuple(Sheet):
     def dive(self):
         push_pyobj(joinSheetnames(self.name, self.cursorRow[0]), self.cursorRow[1])
 
+SheetNamedTuple.addCommand(ENTER, 'dive-row', 'dive()')
 
-class SheetDict(Sheet):
-    rowtype = 'items'
-    commands = [
-        Command('e', 'edit()', 'edit contents of current cell', 'modify-edit-cell'),
-        Command(ENTER, 'dive()', 'dive further into Python object', 'python-dive-row')
-    ]
-    def __init__(self, name, src, **kwargs):
-        super().__init__(name, source=src, **kwargs)
+class SheetDict(PythonSheet):
+    rowtype = 'items'  # rowdef: [key, value]  (editable)
+    def __init__(self, name, source, **kwargs): # source is dict()
+        super().__init__(name, source=source, **kwargs)
 
     def reload(self):
         self.columns = [ColumnItem('key', 0)]
@@ -173,6 +195,10 @@ class SheetDict(Sheet):
     def dive(self):
         push_pyobj(joinSheetnames(self.name, self.cursorRow[0]), self.cursorRow[1])
 
+
+SheetDict.addCommand('e', 'edit-cell', 'edit()')
+SheetDict.addCommand(ENTER, 'dive-row', 'dive()')
+
 class ColumnSourceAttr(Column):
     'Use row as attribute name on sheet source'
     def calcValue(self, attrname):
@@ -181,34 +207,33 @@ class ColumnSourceAttr(Column):
         return setattr(self.sheet.source, attrname, value)
 
 # rowdef: attrname
-class SheetObject(Sheet):
+class SheetObject(PythonSheet):
     rowtype = 'attributes'
-    commands = [
-        Command(ENTER, 'v = getattr(source, cursorRow); push_pyobj(joinSheetnames(name, cursorRow), v() if callable(v) else v)', 'dive further into Python object'),
-        Command('e', 'setattr(source, cursorRow, type(getattr(source, cursorRow))(editCell(1))); sheet.cursorRowIndex += 1; reload()', 'edit contents of current cell', 'modify-edit-cell'),
-        Command('v', 'options.pyobj_show_hidden = not options.pyobj_show_hidden; reload()', 'toggle whether hidden properties are shown'),
-        Command('gv', 'options.pyobj_show_hidden = options.pyobj_show_methods = True; reload()', 'show methods and hidden properties'),
-        Command('zv', 'options.pyobj_show_hidden = options.pyobj_show_methods = False; reload()', 'hide methods and hidden properties'),
-    ]
     def __init__(self, name, obj, **kwargs):
         super().__init__(name, source=obj, **kwargs)
 
     def reload(self):
         self.rows = []
+        vislevel = options.visibility
         for r in dir(self.source):
             try:
-                if options.pyobj_show_hidden or not r.startswith('_'):
-                    self.addRow(r)
-                elif options.pyobj_show_methods or not callable(getattr(self.source, r)):
-                    self.addRow(r)
+                if vislevel <= 1 and r.startswith('_'): continue
+                if vislevel <= 0 and callable(getattr(self.source, r)): continue
+                self.addRow(r)
             except Exception:
                 pass
 
         self.columns = [
             Column(type(self.source).__name__ + '_attr'),
             ColumnSourceAttr('value'),
-            ColumnExpr('docstring', 'value.__doc__')
+            ColumnExpr('docstring', 'value.__doc__ if callable(value) else type(value).__name__')
         ]
         self.recalc()
 
-        self.keyCols = self.columns[0:1]
+        self.setKeys(self.columns[0:1])
+
+SheetObject.addCommand(ENTER, 'dive-row', 'v = getattr(source, cursorRow); push_pyobj(joinSheetnames(name, cursorRow), v() if callable(v) else v)')
+SheetObject.addCommand('e', 'edit-cell', 'setattr(source, cursorRow, type(getattr(source, cursorRow))(editCell(1))); sheet.cursorRowIndex += 1; reload()')
+SheetObject.addCommand('v', 'visibility', 'options.set("visibility", 0 if options.visibility else 2, sheet); reload()')
+SheetObject.addCommand('gv', 'show-hidden', 'options.visibility = 2; reload()')
+SheetObject.addCommand('zv', 'hide-hidden', 'options.visibility -= 1; reload()')
