@@ -1,7 +1,7 @@
 from visidata import *
 
 
-globalCommand('T', 'vd.push(TransposeSheet(name+"_T", source=sheet))', 'open new sheet with rows and columns transposed', 'data-transpose')
+Sheet.addCommand('T', 'transpose', 'vd.push(TransposeSheet(name+"_T", source=sheet))')
 
 
 # rowdef: Column
@@ -13,10 +13,10 @@ class TransposeSheet(Sheet):
             Column('_'.join(c.name for c in self.source.keyCols),
                    getter=lambda c,origcol: origcol.name)
         ]
-        self.keyCols = self.columns[:]
+        self.setKeys(self.columns)
 
         # rows become columns
-        for row in Progress(self.source.rows):
+        for row in Progress(self.source.rows, 'transposing'):
             self.addColumn(Column('_'.join(self.source.rowkey(row)),
                                   getter=lambda c,origcol,row=row: origcol.getValue(row)))
 
